@@ -425,15 +425,13 @@ job_source_closed_callback (GVfsJobSource *job_source,
 }
 
 static void
-re_register_jobs_cb (GVfsDBusMountTracker *proxy,
+re_register_jobs_cb (GVfsBackend *backend,
                      GAsyncResult *res,
                      gpointer user_data)
 {
   GError *error = NULL;
 
-  gvfs_dbus_mount_tracker_call_register_mount_finish (proxy,
-                                                      res,
-                                                      &error);
+  g_vfs_backend_register_mount_finish (backend, res, &error);
   g_debug ("re_register_jobs_cb, error: %p\n", error);
   g_clear_error (&error);
 }
@@ -1069,8 +1067,6 @@ g_vfs_daemon_get_blocking_processes (GVfsDaemon *daemon)
   GArray *processes;
   GList *l;
 
-  g_mutex_lock (&daemon->lock);
-
   processes = g_array_new (FALSE, FALSE, sizeof (GPid));
   for (l = daemon->job_sources; l != NULL; l = l->next)
     {
@@ -1082,8 +1078,6 @@ g_vfs_daemon_get_blocking_processes (GVfsDaemon *daemon)
         }
     }
 
-  g_mutex_unlock (&daemon->lock);
-
   return processes;
 }
 
@@ -1128,12 +1122,8 @@ g_vfs_daemon_close_active_channels (GVfsDaemon *daemon,
 {
   GList *l;
 
-  g_mutex_lock (&daemon->lock);
-
-  for (l = daemon->job_sources; l != NULL; l = l->next)
-    if (G_VFS_IS_CHANNEL (l->data) &&
-        g_vfs_channel_get_backend (G_VFS_CHANNEL (l->data)) == backend)
-      g_vfs_channel_force_close (G_VFS_CHANNEL (l->data));
-
-  g_mutex_unlock (&daemon->lock);
+   for (l = daemon->job_sources; l != NULL; l = l->next)
+      if (G_VFS_IS_CHANNEL (l->data) &&
+          g_vfs_channel_get_backend (G_VFS_CHANNEL (l->data)) == backend)
+        g_vfs_channel_force_close (G_VFS_CHANNEL (l->data));
 }
